"""A script for downloading and analyzing articles from the Journal of Cheminformatics."""

import datetime
import urllib.error
from operator import itemgetter
from pathlib import Path
from textwrap import shorten
from typing import Iterable

import click
import dateutil.parser
import ebooklib
import pandas as pd
import requests
from bs4 import BeautifulSoup
from ebooklib import epub
from tabulate import tabulate
from tqdm.auto import tqdm, trange
from tqdm.contrib.concurrent import process_map
from tqdm.contrib.logging import logging_redirect_tqdm

from autoreviewer.api import review
from autoreviewer.utils import MODULE, strip

HERE = Path(__file__).parent.resolve()
DOI_TO_GITHUB_PATH = HERE.joinpath("doi_to_github.tsv")
ANALYSIS_PATH = HERE.joinpath("analysis.tsv")

DOI_PREFIX = "10.1186/"
JCHEMINF_DOI_PREFIX = "https://doi.org/10.1186/"


def get_epub_url(luid: str) -> str:
    """Get the download link based on a LUID."""
    luid = luid.removeprefix(DOI_PREFIX)  # in case a DOI is passed
    return f"https://jcheminf.biomedcentral.com/counter/epub/10.1186/{luid}.epub"


def get_jcheminf_epub(doi: str) -> epub.EpubBook:
    """Get an ePub object from Journal of Cheminformatics."""
    luid = doi.removeprefix(DOI_PREFIX)
    url = get_epub_url(luid)
    path = MODULE.ensure("epubs", url=url, download_kwargs=dict(progress_bar=False))
    return epub.read_epub(path, options=dict(ignore_ncx=True))


def get_title(book: epub.EpubBook) -> str:
    """Get the title of the article."""
    for item in book.get_items_of_type(ebooklib.ITEM_DOCUMENT):
        soup = BeautifulSoup(item.get_body_content(), "html.parser")
        title_div = soup.find(**{"class": "ArticleTitle"})
        if title_div:
            return strip(remove_non_ascii(title_div.text))
    raise ValueError


def _get_date(book: epub.EpubBook) -> datetime.date | None:
    """Get the title of the article."""
    for element in find_class(book, "HistoryDate"):
        return dateutil.parser.parse(remove_non_ascii(element.text))
    return None


def get_date(book: epub.EpubBook) -> str:
    """Get the date of the article in YYYY-MM-DD."""
    d = _get_date(book)
    if d:
        return d.strftime("%Y-%m-%d")
    return ""


def get_year(book: epub.EpubBook) -> str:
    """Get the year of the article."""
    d = _get_date(book)
    if d:
        return d.strftime("%Y")
    return ""


def find_class(book: epub.EpubBook, name: str):
    """Get the title of the article."""
    for item in book.get_items_of_type(ebooklib.ITEM_DOCUMENT):
        soup = BeautifulSoup(item.get_body_content(), "html.parser")
        if element := soup.find(**{"class": name}):
            yield element


def get_github(book: epub.EpubBook) -> Iterable[str]:
    """Iterate over the GitHub references in the "Availability of data and materials" section."""
    for item in book.get_items_of_type(ebooklib.ITEM_DOCUMENT):
        soup = BeautifulSoup(item.get_body_content(), "html.parser")
        data_availability = soup.find(**{"class": "DataAvailability"})
        if not data_availability:
            continue

        text = (
            data_availability.get_text(strip=False).strip().replace("‚Äč", "").replace("\u200b", "")
        )
        text_cleaned = (
            remove_non_ascii(text).removeprefix("Availability of data and materials").strip()
        )
        count = 0
        for part in text_cleaned.split():
            part_lower = part.lower()
            if "github.com" in part_lower:
                count += 1
                yv = strip(part_lower.split("github.com")[1])
                yield "/".join(yv.split("/")[:2]).split(".")[0]
            # elif "bitbucket.org" in part_lower:
            #     count += 1
            #     yv = strip(part_lower.split("bitbucket.org")[1])
            #     yield "bitbucket", "/".join(yv.split("/")[:2])
            else:
                continue
        if not count:
            tqdm.write(
                click.style("No GitHub found for ", fg="yellow")
                + click.style(f"{get_title(book)} ({get_year(book)})", fg="yellow", bold=True)
                + "\n"
                + text_cleaned.replace("\n", " ").replace("  ", " ")
                + "\n"
            )


def remove_non_ascii(string: str) -> str:
    """Remove all non-ASCII characters from a string."""
    return string.encode("ascii", errors="ignore").decode()


def _process(doi: str) -> tuple[str, str, str | None, str | None]:
    with logging_redirect_tqdm():
        try:
            book = get_jcheminf_epub(doi)
        except (epub.EpubException, urllib.error.HTTPError):
            return doi, "", None, None
        else:
            repos = [r for r in get_github(book) if r]  # TODO multiple checks per repo later
            return doi, get_date(book), get_title(book), repos[0] if repos else None


def scrape_dois(top: int = 28) -> list[str]:
    """Scrape the list of DOIs from the Journal of Cheminformatics' articles page."""
    url = "https://jcheminf.biomedcentral.com/articles?searchType=journalSearch&sort=PubDate&page="
    dois: set[str] = set()
    for i in trange(1, top + 1, unit="page", desc="Scraping JChemInf site"):
        res = requests.get(url + str(i))
        soup = BeautifulSoup(res.text, "html.parser")
        for element in soup.find_all(**{"class": "c-listing__item"}):
            a = element.find(**{"data-test": "title-link"})
            if a is None:
                continue
            title = a.get_text()

            article_type = element.find(**{"data-test": "result-list"})
            article_type_text = article_type.get_text()
            if article_type_text in {"Editorial", "Review"}:
                tqdm.write(f"Skipping {article_type_text}: {title}")
                continue

            doi = a.attrs["href"].removeprefix("/articles/")
            dois.add(doi)
    return sorted(dois)


SKIP_REPOS = {
    "shenggenglin/mddi-scl",
    "duaibeom/molfindergithubrepository",
    "awslabs/dgl-livesci",
}


@click.command()
@click.option("--reindex", is_flag=True, help="If true, reindex papers")
def main(reindex: bool) -> None:
    """Run the analysis."""
    if DOI_TO_GITHUB_PATH.is_file() and not reindex:
        df = pd.read_csv(DOI_TO_GITHUB_PATH, sep="\t")
    else:
        with logging_redirect_tqdm():
            dois = scrape_dois()
        rv = process_map(_process, dois, desc="processing ePubs", unit="article", chunksize=20)
        rv = sorted(set(rv), key=itemgetter(1), reverse=True)  # sort by date

        columns = ["doi", "date", "title", "github"]
        df = pd.DataFrame(rv, columns=columns).drop_duplicates()
        click.echo(f"Writing to {DOI_TO_GITHUB_PATH}")
        df.to_csv(DOI_TO_GITHUB_PATH, sep="\t", index=False)
        click.echo(
            tabulate(
                [
                    (doi.removeprefix("10.1186/"), date, shorten(title, 60), shorten(github, 60))
                    for doi, date, title, github in rv
                    if github
                ],
                headers=columns,
                tablefmt="github",
            )
        )
        length = len(rv)
        has_epub = sum(bool(row[1]) for row in rv)
        has_github = sum(bool(row[3]) for row in rv)
        click.echo(
            f"Retrieved ePubs for {has_epub:,}/{length:,} ({has_epub / length:.1%}) "
            f"and GitHub repos for {has_github:,}/{length:,} ({has_github / length:.1%})"
        )

    rows = []
    for doi, date, title, repo in tqdm(df.values, desc="Loading cached", unit="repo"):
        row = {"doi": doi, "date": date, "title": title}
        if pd.isna(repo):
            rows.append(row)
            continue

        if "." in repo:  # ends with .git
            repo = repo.split(".")[0]

        if repo in SKIP_REPOS:
            # so broken we have to skip
            rows.append({"doi": doi, "date": date, "title": title})
            continue
        try:
            owner, name = repo.split("/")
        except ValueError:
            tqdm.write(f"Failed: {repo}")
            rows.append(row)
            continue
        try:
            results = review(owner, name)
        except Exception:
            tqdm.write(f"Failed: {repo}")
            rows.append(row)
            continue

        row.update(results.get_dict())
        rows.append(row)

    df_full = pd.DataFrame(rows)
    click.echo(f"Writing to {ANALYSIS_PATH}")
    df_full.to_csv(ANALYSIS_PATH, sep="\t", index=False)


if __name__ == "__main__":
    main()



"""Utilities."""

import hashlib
import json
import subprocess
from contextlib import redirect_stderr, redirect_stdout
from functools import lru_cache
from pathlib import Path
from typing import Any, Optional

import pystow
import requests
from pyroma import projectdata, ratings
from ratelimit import rate_limited
from tqdm import tqdm
from tqdm.contrib.logging import logging_redirect_tqdm

#: Wikidata SPARQL endpoint. See https://www.wikidata.org/wiki/Wikidata:SPARQL_query_service#Interfacing
WIKIDATA_ENDPOINT = "https://query.wikidata.org/bigdata/namespace/wdq/sparql"

#: The module where JCheminf stuff goes
MODULE = pystow.module("jcheminf")


def strip(s: str) -> str:
    """Strip bad characters."""
    for t in ".,\\/()[]{}_":
        s = s.strip(t)
    return s


@rate_limited(calls=5_000, period=60 * 60)
def github_api(
    url: str,
    accept: Optional[str] = None,
    params: Optional[dict[str, Any]] = None,
    token: str | None = None,
) -> requests.Response:
    """Request an endpoint from the GitHub API."""
    if token is None:
        # Load the GitHub access token via PyStow. We'll
        # need it, so we don't hit the rate limit
        token = pystow.get_config("github", "token", raise_on_missing=True)
    headers = {
        "Authorization": f"token {token}",
    }
    if accept:
        headers["Accept"] = accept
    return requests.get(url, headers=headers, params=params)


@lru_cache
def get_repo_metadata(repo: str) -> dict:
    """Get repository metadata."""
    md5 = hashlib.md5()
    md5.update(repo.encode("utf-8"))
    path = MODULE.join("github-info", name=f"{md5.hexdigest()[:8]}.json")
    if path.is_file():
        return json.loads(path.read_text())
    res_json = github_api(f"https://api.github.com/repos/{repo}").json()
    path.write_text(json.dumps(res_json, indent=2))
    return res_json


ResTup = tuple[str, str] | tuple[None, None]


def get_file(
    repo: str, name: str | list[str], *, branch: str = "main", desc: str | None = None
) -> ResTup:
    """Get the file name and text, if available."""
    if isinstance(name, str):
        name = [name]

    owner, repo_name = repo.split("/")
    directory = pystow.join("github", owner, repo_name)
    if directory.exists() and any(directory.iterdir()):
        for n in name:
            p = directory.joinpath(n)
            if p.is_file():
                return n, p.read_text()
        return None, None

    base_url = f"https://raw.githubusercontent.com/{repo}/{branch}"
    for n in tqdm(name, leave=False, desc=desc, disable=True):
        url = f"{base_url}/{n}"
        with logging_redirect_tqdm():
            res = requests.get(
                url, timeout=1
            )  # timeout is short since these are small, simple files
        if res.status_code == 200:
            return n, res.text
    return None, None


@lru_cache
def get_readme(repo: str, branch: str = "main") -> ResTup:
    """Get the readme file name and text, if available."""
    return get_file(
        repo, branch=branch, name=["README.md", "README.rst", "README.txt"], desc="Finding README"
    )


@lru_cache
def get_setup_config(repo: str, branch: str = "main") -> ResTup:
    """Get the setup configuration file name and text, if available."""
    return get_file(
        repo,
        branch=branch,
        name=["setup.cfg", "setup.py", "pyproject.toml"],
        desc="Finding setup conf.",
    )


@lru_cache
def get_license_file(repo: str, branch: str = "main") -> ResTup:
    """Get the license file name and text, if available."""
    return get_file(
        repo, branch=branch, name=["LICENSE", "LICENSE.md", "LICENSE.rst"], desc="Finding license"
    )


def get_repo_path(owner: str, repo: str) -> Path | None:
    """Clone a repository from GitHub locally inside the PyStow folder."""
    directory = pystow.join("github", owner, repo)
    if directory.is_dir():
        if any(directory.iterdir()):  # check not empty
            return directory
    url = f"https://github.com/{owner}/{repo}"
    try:
        subprocess.check_call(["git", "clone", "--depth", "1", url, directory.as_posix()])
    except subprocess.CalledProcessError:
        return None
    return directory


def check_black(directory: str | Path) -> bool:
    """Check if the folder passes ``black --check``."""
    directory = Path(directory).resolve()
    parts = ["black", "--check", "--quiet", directory.as_posix()]
    try:
        subprocess.check_call(parts, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        return False
    else:
        return True


def remote_check_black_github(owner, repo) -> bool:
    """Check if the GitHub repository passes ``black --check``."""
    directory = get_repo_path(owner, repo)
    if directory is None:
        raise RuntimeError
    return check_black(directory)


def get_has_issues(owner: str, name: str) -> bool:
    """Check if the GitHub repository has issues enabled."""
    res = get_repo_metadata(f"{owner}/{name}")
    return res["has_issues"]


def get_default_branch(owner: str, name: str) -> str:
    """Get the default branch for a GitHub repository."""
    res = get_repo_metadata(f"{owner}/{name}")
    return res["default_branch"]


def get_programming_language(owner: str, name: str) -> str:
    """Get the primary programing language."""
    res = get_repo_metadata(f"{owner}/{name}")
    return res["language"]


def get_is_fork(owner: str, name: str) -> bool:
    """Get if the repository is a fork."""
    res = get_repo_metadata(f"{owner}/{name}")
    return res["fork"]


def get_license(owner: str, name: str) -> str | None:
    """Get the license SPDX identifier, if available."""
    res = get_repo_metadata(f"{owner}/{name}")
    repo_license = (res.get("license") or {}).get("spdx_id")
    if repo_license == "NOASSERTION":
        return "Unknown"
    return repo_license


def check_pyroma(path: str | Path) -> tuple[int, list[str]]:
    """Return feedback from ``pyroma`` or None if passing."""
    path = Path(path).resolve()
    try:
        with redirect_stdout(None), redirect_stderr(None):
            data = projectdata.get_data(path.as_posix())
            rv = ratings.rate(data)
    except Exception:
        return 0, []
    else:
        return rv


def remote_check_pyroma(owner: str, name: str) -> tuple[int, list[str]]:
    """Check if the GitHub repository passes ``pyroma``."""
    directory = get_repo_path(owner, name)
    if directory is None:
        return 0, []
    return check_pyroma(directory)


def check_no_scripts(owner: str, name: str) -> list[str]:
    """Get scripts sitting in the home directory."""
    directory = get_repo_path(owner, name)
    if directory is None:
        return []
    scripts = directory.glob("*.py")
    skips = {"setup.py"}
    return [s.stem for s in scripts if s.name not in skips]



# -*- coding: utf-8 -*-

"""Command line interface for :mod:`autoreviewer`.

Why does this file exist, and why not put this in ``__main__``? You might be tempted to import things from ``__main__``
later, but that will cause problems--the code will get executed twice:

- When you run ``python3 -m autoreviewer`` python will execute``__main__.py`` as a script.
  That means there won't be any ``autoreviewer.__main__`` in ``sys.modules``.
- When you import __main__ it will get executed again (as a module) because
  there's no ``autoreviewer.__main__`` in ``sys.modules``.

.. seealso:: https://click.palletsprojects.com/en/7.x/setuptools/#setuptools-integration
"""

import logging
import subprocess
from pathlib import Path

import click

from autoreviewer import review

__all__ = [
    "main",
]

logger = logging.getLogger(__name__)


@click.command()
@click.argument("name_or_url")
@click.option(
    "--path",
    type=click.Path(),
    help="A custom path to the output file, otherwise outputs in the current directory",
)
@click.option("--open", is_flag=True, help="If set, opens the PDF")
def main(name_or_url: str, path: Path | None, open: bool):
    """CLI for autoreviewer."""
    owner, repo, *_ = (
        name_or_url.removeprefix("https://github.com/").removesuffix(".git").rstrip("/").split("/")
    )
    repo_clean = repo.lower().replace("_", "-")
    results = review(owner, repo)
    if path is not None:
        results.write_pandoc(path)
    else:
        path = Path.cwd().joinpath(f"{repo_clean}-review.pdf")
        results.write_pandoc(path)
        click.echo(f"Wrote review PDF to {path}")
    if open:
        subprocess.call(["open", path.as_posix()])


if __name__ == "__main__":
    main()



"""Generate summary charts."""

import datetime

import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from matplotlib.ticker import FuncFormatter


def _p_status(x):
    if pd.isna(x):
        return "No Repo"
    elif x <= 0:
        return False
    else:
        return True


def _license_status(x):
    if pd.isna(x):
        return "No License"
    elif x == "Unknown":
        return "Unknown"
    else:
        return "Present"


def _percentage(df, column):
    series = df.groupby("year")[column].mean()
    rdf = series.to_frame().reset_index()
    return rdf


def _percentage_axis(ax):
    formatter = FuncFormatter(lambda y, _: "{:.1%}".format(y))
    ax.yaxis.set_major_formatter(formatter)


def main():
    """Generate summary charts."""
    fig, axes = plt.subplots(2, 3, figsize=(11, 5))

    axes = axes.ravel()

    today = datetime.date.today()
    df = pd.read_csv("analysis.tsv", sep="\t")
    df["has_github"] = df["repo"].notna()
    df["date"] = pd.to_datetime(df["date"])
    df = df[df["date"].notna()]
    df["year"] = df["date"].dt.year
    # 2017 was the first year, where a repository was detected
    df = df[(2017 < df["year"]) & (df["year"] < today.year)]  # don't make ragged chart

    sns.barplot(x="year", y="has_github", data=_percentage(df, "has_github"), ax=axes[0])
    axes[0].set_ylabel("")
    axes[0].set_xlabel("")
    axes[0].set_title("Has GitHub Repo")
    _percentage_axis(axes[0])

    sns.barplot(
        x="year", y="has_issues", data=_percentage(df[df["has_github"]], "has_issues"), ax=axes[1]
    )
    axes[1].set_ylabel("")
    axes[1].set_xlabel("")
    axes[1].set_title("Has Issue Tracker")
    _percentage_axis(axes[1])

    df["package_status"] = df["pyroma_score"].map(_p_status)
    sns.barplot(
        x="year",
        y="package_status",
        data=_percentage(df[df["has_github"]], "package_status"),
        ax=axes[2],
    )
    axes[2].set_xlabel("")
    axes[2].set_ylabel("")
    axes[2].set_title("Packaged Code")
    _percentage_axis(axes[2])

    sns.barplot(
        x="year",
        y="is_blackened",
        data=_percentage(df[df["has_github"]], "is_blackened"),
        ax=axes[3],
    )
    axes[3].set_xlabel("")
    axes[3].set_ylabel("")
    axes[3].set_title("Applied Linting")
    _percentage_axis(axes[3])

    sns.barplot(
        x="year",
        y="has_installation_docs",
        data=_percentage(df[df["has_github"]], "has_installation_docs"),
        ax=axes[4],
    )
    axes[4].set_xlabel("")
    axes[4].set_ylabel("")
    axes[4].set_title("Has Installation Documentation")
    _percentage_axis(axes[4])

    df["license_status"] = df["license"].map(_license_status)
    sns.countplot(x="year", hue="license_status", data=df[df["has_github"]], ax=axes[5])
    axes[5].set_xlabel("")
    axes[5].set_ylabel("")
    axes[5].set_title("License Status")

    plt.suptitle("Analysis of J. Chem. Inf. Papers", fontsize=16)
    plt.tight_layout()
    plt.savefig("jcheminf_summary.png", dpi=300)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""Trivial version test."""

import unittest

from autoreviewer.version import get_version


class TestVersion(unittest.TestCase):
    """Trivially test a version."""

    def test_version_type(self):
        """Test the version is a string.

        This is only meant to be an example test.
        """
        version = get_version()
        self.assertIsInstance(version, str)



# -*- coding: utf-8 -*-

"""Main code."""

import dataclasses
import datetime
import os
from dataclasses import dataclass, field
from pathlib import Path

import click
from jinja2 import Environment, FileSystemLoader
from pystow.utils import get_commit
from tqdm import tqdm

from autoreviewer.utils import (
    check_no_scripts,
    get_default_branch,
    get_has_issues,
    get_is_fork,
    get_license,
    get_programming_language,
    get_readme,
    get_setup_config,
    remote_check_black_github,
    remote_check_pyroma,
)

HERE = Path(__file__).parent.resolve()
TEMPLATES = HERE.joinpath("templates")

environment = Environment(
    autoescape=True, loader=FileSystemLoader(TEMPLATES), trim_blocks=True, lstrip_blocks=True
)
review_template = environment.get_template("review.md")


@dataclass
class Results:
    """Results from analysis.

    1. Does the repository contain a LICENSE file in its root?
    2. Does the repository contain a README file in its root?
    3. Does the repository contain an associated public issue tracker?
    4. Has the repository been externally archived on Zenodo, FigShare, or equivalent that is referenced in the README?
    5. Does the README contain installation documentation?
    6. Is the code from the repository installable in a straight-forward manner?
    7. Does the code conform to an external linter (e.g., black for Python)?
    """

    owner: str
    name: str
    language: str
    license_name: str | None
    has_zenodo: bool
    has_setup: bool
    has_installation_docs: bool
    has_issues: bool
    is_fork: bool
    is_blackened: bool
    pyroma_score: int
    pyroma_failures: list[str]
    commit: str
    branch: str
    root_scripts: list[str]

    date: datetime.date = field(default_factory=datetime.date.today)
    readme_type: str | None = None

    def get_dict(self):
        """Get this review as a dict."""
        d = dataclasses.asdict(self)
        d["repo"] = self.repo_url
        d["license"] = d.pop("license_name")
        d["commit"] = d.pop("commit")[:8]
        del d["owner"]
        del d["name"]
        del d["pyroma_failures"]
        del d["date"]
        return d

    @property
    def has_readme(self) -> bool:
        """Get if there is a README."""
        return self.readme_type is not None

    @property
    def repo(self) -> str:
        """Get the full repo."""
        return f"{self.owner}/{self.name}"

    @property
    def repo_url(self) -> str:
        """Get the repo URL."""
        return f"https://github.com/{self.repo}"

    @property
    def has_license(self) -> bool:
        """Get if the license exists."""
        return self.license_name is not None

    @property
    def passes(self) -> bool:
        """Return if all checks have passed."""
        return all(
            [
                self.has_issues,
                self.has_license,
                self.has_readme,
                self.has_zenodo,
                self.has_setup,
                self.has_installation_docs,
                self.is_blackened,
            ]
        )

    def render(self) -> str:
        """Render the template for GitHub issues."""
        return review_template.render(
            repo=self.repo,
            repo_url=f"https://github.com/{self.repo}",
            name=self.name,
            branch=self.branch,
            license_name=self.license_name,
            has_readme=self.has_readme,
            has_zenodo=self.has_zenodo,
            has_setup=self.has_setup,
            root_scripts=self.root_scripts,
            has_installation_docs=self.has_installation_docs,
            readme_type=self.readme_type,
            has_issues=self.has_issues,
            pyroma_score=self.pyroma_score,
            pyroma_failures=self.pyroma_failures,
            date=self.date.strftime("%Y-%m-%d"),
            commit=self.commit,
            passes=self.passes,
            is_blackened=self.is_blackened,
            issue=None,  # FIXME
        )

    def write_pandoc(self, path: str | Path) -> None:
        """Write to a PDF with Pandoc."""
        path = Path(path).resolve()
        markdown_path = path.with_suffix(".md")
        markdown_path.write_text(self.render())
        click.echo(f"Wrote review markdown to {markdown_path}")
        command = (
            f"pandoc {markdown_path.as_posix()} -o {path.as_posix()} -V colorlinks=true -V "
            "linkcolor=blue -V urlcolor=blue -V toccolor=gray"
        )
        click.echo(command)
        os.system(command)


README_MAP = {"README.md": "markdown", "README.rst": "rst", "README.txt": "txt", None: None}


def _has_markdown_installation(text: str | None) -> bool:
    if not text:
        return False
    for line in text.splitlines():
        if line.startswith("#") and "installation" in line.lower():
            return True
    return False


def review(owner: str, name: str) -> Results:
    """Review a repository."""
    repo = f"{owner}/{name}"
    branch = get_default_branch(owner, name)
    is_blackened = remote_check_black_github(owner, name)

    readme_name, readme_text = get_readme(repo=repo, branch=branch)
    readme_type = README_MAP[readme_name]
    if readme_type is None:
        has_zenodo = False
        has_installation_docs = False
    elif readme_type == "markdown":
        has_zenodo = readme_text is not None and (
            "https://zenodo.org/badge/DOI/10.5281/" in readme_text
            or "https://zenodo.org/badge/latestdoi/" in readme_text
        )
        has_installation_docs = _has_markdown_installation(readme_text)
    elif readme_type == "rst":
        has_zenodo = False
        has_installation_docs = False
        tqdm.write(f"README was RST, assuming missing zenodo/installation docs: {repo}")
    elif readme_type == "txt":
        has_zenodo = False
        has_installation_docs = False
        tqdm.write(f"README was TXT, assuming missing zenodo/installation docs: {repo}")
    else:
        raise TypeError

    setup_name, setup_text = get_setup_config(repo=repo, branch=branch)
    has_setup = setup_name is not None

    license_name = get_license(owner, name)

    pyroma_score, pyroma_failures = remote_check_pyroma(owner, name)
    has_issues = get_has_issues(owner, name)
    language = get_programming_language(owner, name)
    is_fork = get_is_fork(owner, name)
    commit = get_commit(owner, name)

    root_scripts = check_no_scripts(owner, name)

    return Results(
        owner=owner,
        name=name,
        language=language,
        license_name=license_name,
        readme_type=readme_type,
        has_installation_docs=has_installation_docs,
        has_zenodo=has_zenodo,
        has_issues=has_issues,
        is_fork=is_fork,
        is_blackened=is_blackened,
        pyroma_score=pyroma_score,
        pyroma_failures=pyroma_failures,
        has_setup=has_setup,
        root_scripts=root_scripts,
        commit=commit,
        branch=branch,
    )


if __name__ == "__main__":
    review("jonghyunlee1993", "DLM-DTI_hint-based-learning").write_pandoc(
        HERE.joinpath("review.pdf")
    )
